@@ -5,7 +5,8 @@ import unittest
 import zipfile
 from xml.etree import ElementTree
 from elifetools import parseJATS as parser
-from elifecleaner import LOGGER, configure_logging, transform
+from elifetools import xmlio
+from elifecleaner import LOGGER, configure_logging, transform, zip_lib
 from elifecleaner.transform import ArticleZipFile, WELLCOME_FUNDING_STATEMENT
 from tests.helpers import delete_files_in_folder, read_fixture
 
@@ -223,6 +224,8 @@ class TestTransform(unittest.TestCase):
 
 
 class TestXmlElementToString(unittest.TestCase):
+    "tests for xml_element_to_string()"
+
     def test_xml_element_to_string(self):
         if sys.version_info < (3, 8):
             # pre Python 3.8 tag attributes are automatically alphabetised
@@ -246,6 +249,33 @@ class TestXmlElementToString(unittest.TestCase):
         expected = '<?xml version="1.0" ?>%s' % xml_string
         self.assertEqual(transform.xml_element_to_string(root), expected)
 
+    def test_all_arguments(self):
+        "test using doctype argument and others"
+        xml_string = "<article/>"
+        qualifiedName = "article"
+        publicId = (
+            "-//NLM//DTD JATS (Z39.96) Journal Archiving and"
+            " Interchange DTD v1.3 20210610//EN"
+        )
+        systemId = "JATS-archivearticle1.dtd"
+        internalSubset = None
+        encoding = "UTF-8"
+        processing_instructions = None
+        doctype = xmlio.build_doctype(qualifiedName, publicId, systemId, internalSubset)
+        root = ElementTree.fromstring(xml_string)
+        expected = '<?xml version="1.0" ?><!DOCTYPE %s PUBLIC "%s"  "%s">%s' % (
+            qualifiedName,
+            publicId,
+            systemId,
+            xml_string,
+        )
+        # invoke
+        result = transform.xml_element_to_string(
+            root, doctype, encoding, processing_instructions
+        )
+        # assert
+        self.assertEqual(result, expected)
+
 
 class TestCodeFileList(unittest.TestCase):
     def test_code_file_list(self):
@@ -298,6 +328,117 @@ class TestZipCodeFile(unittest.TestCase):
         self.assertEqual(str(to_file), str(expected))
 
 
+class TestCoverArtFileList(unittest.TestCase):
+    def test_cover_art_file_list(self):
+        xml_string = read_fixture("cover_art_file_list.xml")
+        expected = read_fixture("cover_art_file_list.py")
+        root = ElementTree.fromstring(xml_string)
+        cover_art_files = transform.cover_art_file_list(root)
+        self.assertEqual(cover_art_files, expected)
+
+
+class TestTransformCoverArtFiles(unittest.TestCase):
+    def setUp(self):
+        self.temp_dir = "tests/tmp"
+        self.zip_file_base = "30-01-2019-RA-eLife-45644"
+        self.zip_file_name = "%s.zip" % self.zip_file_base
+        zip_file = "tests/test_data/%s" % self.zip_file_name
+        self.asset_file_name_map = zip_lib.unzip_zip(zip_file, self.temp_dir)
+        xml_file_name = "%s.xml" % self.zip_file_base
+        self.xml_file_path = os.path.join(
+            self.temp_dir, "30-01-2019-RA-eLife-45644", xml_file_name
+        )
+
+    def tearDown(self):
+        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
+
+    def test_transform_cover_art_files(self):
+        "test transforming cover art file name and modifying the XML"
+        old_file_name = "Potential striking image.tif"
+        new_file_name = "45644-a_striking_image.tif"
+        file_transformations = []
+        from_file = ArticleZipFile(
+            old_file_name,
+            "%s/%s" % (self.zip_file_base, old_file_name),
+            "%s%s/%s" % (self.temp_dir, self.zip_file_base, old_file_name),
+        )
+        to_file = ArticleZipFile(
+            new_file_name,
+            "%s/%s" % (self.zip_file_base, new_file_name),
+            "%s%s/%s" % (self.temp_dir, self.zip_file_base, new_file_name),
+        )
+        file_transformations.append((from_file, to_file))
+        result = transform.transform_cover_art_files(
+            self.xml_file_path,
+            self.asset_file_name_map,
+            file_transformations,
+            self.zip_file_name,
+        )
+        expected_key = "30-01-2019-RA-eLife-45644/45644-a_striking_image.tif"
+        # assert the new file name is in the new asset_key_map
+        self.assertTrue(expected_key in result.keys())
+        # assert on XML file contents
+        with open(self.xml_file_path, "r") as open_file:
+            self.assertTrue(
+                new_file_name in open_file.read(),
+                "Did not find %s in the XML" % new_file_name,
+            )
+            self.assertFalse(
+                old_file_name in open_file.read(),
+                "Unexpectedly found %s in the XML" % old_file_name,
+            )
+
+
+class TestStrikingImageFileName(unittest.TestCase):
+    def test_striking_image_file_name(self):
+        file_name = "test.tif"
+        article_id = "45644"
+        index = 0
+        expected = "45644-a_striking_image.tif"
+        result = transform.striking_image_file_name(file_name, article_id, index)
+        self.assertEqual(result, expected)
+
+
+class TestCoverArtFileTransformations(unittest.TestCase):
+    def setUp(self):
+        self.temp_dir = "tests/tmp"
+        self.zip_file_base = "30-01-2019-RA-eLife-45644"
+        self.zip_file_name = "%s.zip" % self.zip_file_base
+        zip_file = "tests/test_data/%s" % self.zip_file_name
+        self.article_id = "45644"
+        self.asset_file_name_map = zip_lib.unzip_zip(zip_file, self.temp_dir)
+
+    def tearDown(self):
+        delete_files_in_folder(self.temp_dir, filter_out=[".keepme"])
+
+    def test_cover_art_file_transformations(self):
+        "create a list of file transformations from old striking image file name to new one"
+        old_file_name = "Potential striking image.tif"
+        new_file_name = "45644-a_striking_image.tif"
+        cover_art_files = [{"upload_file_nm": old_file_name}]
+        expected = [
+            (
+                ArticleZipFile(
+                    old_file_name,
+                    "%s/%s" % (self.zip_file_base, old_file_name),
+                    "%s/%s/%s" % (self.temp_dir, self.zip_file_base, old_file_name),
+                ),
+                ArticleZipFile(
+                    new_file_name,
+                    "%s/%s" % (self.zip_file_base, new_file_name),
+                    "%s/%s/%s" % (self.temp_dir, self.zip_file_base, new_file_name),
+                ),
+            )
+        ]
+        result = transform.cover_art_file_transformations(
+            cover_art_files,
+            self.asset_file_name_map,
+            self.article_id,
+            self.zip_file_name,
+        )
+        self.assertEqual(str(result), str(expected))
+
+
 class TestFromFileToFileMap(unittest.TestCase):
     def test_from_file_to_file_map(self):
         from_xml_name = "source.c"
@@ -311,6 +452,98 @@ class TestFromFileToFileMap(unittest.TestCase):
         )
 
 
+class TestTransformSubjectTags(unittest.TestCase):
+    "tests for transform.transform_subject_tags()"
+
+    def test_transform_subject_tags(self):
+        "test removing (VOR) from a value and removing duplicate subject tags"
+        xml_string = (
+            "<article>"
+            "<front>"
+            "<article-meta>"
+            "<article-categories>"
+            '<subj-group subj-group-type="display-channel">'
+            "<subject>Research Article (VOR)</subject>"
+            "</subj-group>"
+            '<subj-group subj-group-type="heading">'
+            "<subject>Epidemiology and Global Health</subject>"
+            "</subj-group>"
+            '<subj-group subj-group-type="heading">'
+            "<subject>Epidemiology and Global Health</subject>"
+            "</subj-group>"
+            '<subj-group subj-group-type="heading">'
+            "<subject>Medicine</subject>"
+            "</subj-group>"
+            '<subj-group subj-group-type="heading">'
+            "<subject>Medicine</subject>"
+            "</subj-group>"
+            "</article-categories>"
+            "</article-meta>"
+            "</front>"
+            "</article>"
+        )
+        root = ElementTree.fromstring(xml_string)
+        expected = (
+            b"<article>"
+            b"<front>"
+            b"<article-meta>"
+            b"<article-categories>"
+            b'<subj-group subj-group-type="display-channel">'
+            b"<subject>Research Article</subject>"
+            b"</subj-group>"
+            b'<subj-group subj-group-type="heading">'
+            b"<subject>Epidemiology and Global Health</subject>"
+            b"</subj-group>"
+            b'<subj-group subj-group-type="heading">'
+            b"<subject>Medicine</subject>"
+            b"</subj-group>"
+            b"</article-categories>"
+            b"</article-meta>"
+            b"</front>"
+            b"</article>"
+        )
+        transform.transform_subject_tags(root, None)
+        self.assertEqual(ElementTree.tostring(root), expected)
+
+
+class TestTransformKwdTags(unittest.TestCase):
+    "tests for transform.transform_kwd_tags()"
+
+    def test_transform_kwd_tags(self):
+        "test removing duplicate kwd tags"
+        xml_string = (
+            "<article>"
+            "<front>"
+            "<article-meta>"
+            '<kwd-group kwd-group-type="research-organism">'
+            "<title>Research organism</title>"
+            "<kwd>Human</kwd>"
+            "<kwd>Human</kwd>"
+            "<kwd>Other</kwd>"
+            "<kwd>Other</kwd>"
+            "</kwd-group>"
+            "</article-meta>"
+            "</front>"
+            "</article>"
+        )
+        root = ElementTree.fromstring(xml_string)
+        expected = (
+            b"<article>"
+            b"<front>"
+            b"<article-meta>"
+            b'<kwd-group kwd-group-type="research-organism">'
+            b"<title>Research organism</title>"
+            b"<kwd>Human</kwd>"
+            b"<kwd>Other</kwd>"
+            b"</kwd-group>"
+            b"</article-meta>"
+            b"</front>"
+            b"</article>"
+        )
+        transform.transform_kwd_tags(root, None)
+        self.assertEqual(ElementTree.tostring(root), expected)
+
+
 class TestTransformXmlFileTags(unittest.TestCase):
     def test_transform_xml_file_tags(self):
         # populate an ElementTree
@@ -458,6 +691,70 @@ class TestTransformXmlHistoryTags(unittest.TestCase):
         )
         self.assertEqual(transform.xml_element_to_string(root_output), expected)
 
+    def test_keep_sent_for_review_date(self):
+        "keep the sent-for-review date in the history tag"
+        # populate an ElementTree
+        sent_for_review_xml = '<history><date date-type="sent-for-review"><day>15</day><month>04</month><year>2023</year></date></history>'
+        xml_string = (
+            '<article article-type="research-article">'
+            "<front>"
+            "<journal-meta>"
+            '<journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
+            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
+            "</journal-meta>"
+            "<article-meta>"
+            "<article-categories>"
+            '<subj-group subj-group-type="display-channel">'
+            "<subject>Test</subject>"
+            "</subj-group>"
+            "</article-categories>"
+            "<history>"
+            '<date date-type="accepted">'
+            "<day>16</day>"
+            "<month>05</month>"
+            "<year>2023</year>"
+            "</date>"
+            '<date date-type="received">'
+            "<day>22</day>"
+            "<month>12</month>"
+            "<year>2022</year>"
+            "</date>"
+            '<date date-type="rev-recd">'
+            "<day>11</day>"
+            "<month>04</month>"
+            "<year>2023</year>"
+            "</date>"
+            "%s"
+            "</history>"
+            "</article-meta>"
+            "</front>"
+            "</article>"
+        ) % sent_for_review_xml
+        root = ElementTree.fromstring(xml_string)
+        soup = parser.parse_xml(xml_string)
+        # invoke the function
+        root_output = transform.transform_xml_history_tags(root, soup, "test.zip")
+        # find the tag in the XML root returned which will have been altered
+        expected = (
+            '<?xml version="1.0" ?>'
+            '<article article-type="research-article">'
+            "<front>"
+            '<journal-meta><journal-id journal-id-type="nlm-ta">__not_elife__</journal-id>'
+            '<issn publication-format="electronic" pub-type="epub">2050-084X</issn>'
+            "</journal-meta>"
+            "<article-meta>"
+            "<article-categories>"
+            '<subj-group subj-group-type="display-channel">'
+            "<subject>Test</subject>"
+            "</subj-group>"
+            "</article-categories>"
+            "<history>%s</history>"
+            "</article-meta>"
+            "</front>"
+            "</article>"
+        ) % sent_for_review_xml
+        self.assertEqual(transform.xml_element_to_string(root_output), expected)
+
 
 class TestTransformXmlFunding(unittest.TestCase):
     def setUp(self):
